"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const prompts_1 = __importDefault(require("prompts"));
const config_1 = require("../config");
const constants_1 = require("../constants");
const print_1 = require("../print");
const command = (config, ...args) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('Initialize project...');
    const hasExistingConfig = config !== undefined;
    if (hasExistingConfig) {
        (0, print_1.printConfig)(config);
        const { canOverwrite } = yield (0, prompts_1.default)([
            {
                style: 'emoji',
                type: 'toggle',
                name: 'canOverwrite',
                message: 'Config already exists. Do you want to overwrite file?',
                initial: false,
                active: 'yes',
                inactive: 'no'
            }
        ]);
        if (!canOverwrite) {
            console.log('abort!');
            return;
        }
    }
    const { namespace, directory, connectionManagerModuleName, stateManagerModuleName } = yield (0, prompts_1.default)([
        {
            type: 'text',
            name: 'namespace',
            message: 'What is your application name?',
            initial: constants_1.DEFAULT_REVISIONS_NAMESPACE
        },
        {
            type: 'text',
            name: 'directory',
            message: 'Where will you store revision files?',
            initial: constants_1.DEFAULT_REVISIONS_DIRECTORY
        },
        {
            type: 'select',
            name: 'connectionManagerModuleName',
            message: 'What connection manager will you be using?',
            choices: constants_1.REVISIONS_MODULES
                .map(({ moduleName }) => ({
                title: moduleName,
                value: moduleName
            }))
                .concat({
                title: 'No thanks. I want to implement my own.',
                value: 'custom'
            })
        },
        {
            type: 'select',
            name: 'stateManagerModuleName',
            message: 'What state manager will you be using?',
            choices: constants_1.REVISIONS_MODULES
                .map(({ moduleName }) => ({
                title: moduleName,
                value: moduleName
            }))
                .concat({
                title: 'No thanks. I want to implement my own.',
                value: 'custom'
            })
        }
    ]);
    yield (0, config_1.saveConfig)({
        namespace,
        directory,
        connectionManagerModuleName: connectionManagerModuleName === 'custom'
            ? undefined
            : connectionManagerModuleName,
        stateManagerModuleName: stateManagerModuleName === 'custom'
            ? undefined
            : stateManagerModuleName
    });
});
exports.command = command;
