"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const logger_1 = require("./lib/logger");
const migration_1 = require("./service/migration");
const init_1 = require("./command/init");
const new_1 = require("./command/new");
const version_1 = require("./command/version");
const list_1 = require("./command/list");
const up_1 = require("./command/up");
const down_1 = require("./command/down");
const help_1 = require("./command/help");
const print_1 = require("./print");
const config_1 = require("./config");
const loadLocalCommand = (commandName) => {
    const command = {
        init: init_1.command,
        new: new_1.command,
        help: help_1.command
    }[commandName.toLowerCase()];
    return command;
};
const loadRemoteCommand = (commandName) => {
    const command = {
        version: version_1.command,
        list: list_1.command,
        up: up_1.command,
        down: down_1.command
    }[commandName.toLowerCase()];
    return command;
};
const main = () => __awaiter(void 0, void 0, void 0, function* () {
    const args = process.argv.slice(2);
    if (args.length === 0) {
        (0, print_1.printUsage)();
        throw new Error('command required');
    }
    const commandName = args[0];
    const remoteCommand = loadRemoteCommand(commandName);
    const config = yield (0, config_1.loadConfig)();
    if (remoteCommand !== undefined) {
        if (config === undefined) {
            console.error('Missing revisions config.');
            console.error('Please initialize the project using init command.');
            throw new Error('missing revisions config');
        }
        const { connectionManagerModule, stateManagerModule } = config;
        const { getConnectionManager } = connectionManagerModule;
        const { getStateManager } = stateManagerModule;
        const logger = (0, logger_1.getLogger)('CLI');
        // Load connection manager and persistent facade
        const db = getConnectionManager({ logger });
        const state = getStateManager({ logger });
        process.on('exit', () => {
            void db.shutdown();
        });
        const service = new migration_1.MigrationService({ state, logger });
        yield remoteCommand(config, db, state, service, ...args);
    }
    else {
        const localCommand = loadLocalCommand(commandName);
        if (localCommand === undefined) {
            throw new Error('unknown command');
        }
        yield localCommand(config, ...args);
    }
});
exports.main = main;
